
; ************************************************
; eeprom.asm
 #include <p16F84A.inc>
; ************************************************

 UDATA

TEMP_HI	res 1	;00
WDT_H0 	res 1	;01
WDT_H1	res 1	;02
Default res 1	;03
TEMP_LO res 1	;04	
WDT_L0 	res 1	;05	
WDT_L1  res 1	;06

 global	ReadEeprom
 global WriteEeprom
 global DefaultValue
 global EEDefault
 
 global TEMP_HI,WDT_H0,WDT_H1
 global Default,TEMP_LO,WDT_L0,WDT_L1


 CODE

; ************************************************
; this sequence is essential when reading EEPROM

ReadEeprom
	banksel	EECON1
	bsf	EECON1,RD
 	movf	EEDATA,w
 	return

; ************************************************
; this routine writes the data to the location of EEPROM address. 
; the write will not initiate if the below sequence is not exactly 
; followed ( write 55h to EECON2, write AAh to EECON2, then set WR bit)
; check if write is completed? then reset EEIF of EECON1
WriteEeprom
	banksel	INTCON
	bcf	INTCON,GIE
	banksel	EECON1
	bsf	EECON1,WREN
	movlw	0x55
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf	EECON1,WR
	banksel	INTCON
	bsf	INTCON,GIE
	banksel	EECON1
CheckIF	btfss	EECON1,EEIF
	goto	CheckIF	
	bcf	EECON1,EEIF
	return

; ************************************************
; This code reads the data from EEPROM address 00h to 08h 
; and stores it in register located in GPR area 
; if Default is zero then stores default values 

EEDefault
	clrwdt
	banksel	TEMP_HI
	movlw	TEMP_HI		;GPreg, read the values from EEPROM
	movwf	FSR
	banksel	EEADR
	clrf	EEADR
read	call	ReadEeprom
	movwf	INDF
	banksel	EEADR
	incf	EEADR,f
	incf	FSR,f
	btfss	EEADR,3
	goto	read

	banksel	Default
	btfsc	Default,0
	return

DefaultValue
       	movlw	0X55		; 100C High Calibration Temperature point	
	movwf	TEMP_HI
	movlw	0x0F		; 0xXXXX - WDT count
	movwf	WDT_H0
	movlw	0x5A
	movwf	WDT_H1

	movlw	0X19		; 25C Low Calibration Temperature point
	movwf	TEMP_LO
	movlw	0x65		; 0xXXXX - WDT count
	movwf	WDT_L0
	movlw	0x1E
	movwf	WDT_L1
	return
; ************************************************
 
	end


