; *******************************************
; init.asm
; *******************************************
  #include <p16F84A.inc>
 

; *******************************************

 global	InitData
 global	InitWDT
 global InPort
 global OutPort
 global ScanKey
 global	InitTmr0

 global InitInt
 global	IntDelay
 global	Delay1
 global Delay_1
 global Delay20

 global	Switch,Switch1
 extern	Count0,Count1
 extern	Neg,TEMPFlag

	UDATA
Switch		res	1
Switch1		res	1

DB1		res	1
DB2		res 	1
DB3 		res 	1

 CODE

; *******************************************
; Configure the prescaler asigned to WDT 
InitWDT
	banksel	OPTION_REG	; select bank for OPTION_REG 	
				; for WDT as 1:2 prescaler
	movlw	0x09		; PSA=1,PS2:PS1=001 
	movwf	OPTION_REG	; 
	bcf	STATUS,RP0
	return

; *******************************************
; Initialise interrupts

InitInt
	clrwdt
	BANKSEL	OPTION_REG	; select bank for OPTION_REG 	
				; for WDT as 1:1 prescaler
	movlw	0x09		; PSA=1,PS2:PS1=001 for 1:2 WDT rate
	movwf	OPTION_REG
	BANKSEL	TMR0
	clrf	TMR0
	BANKSEL	INTCON
	clrf	INTCON
	bsf	INTCON,T0IE		; TMR0 on
   	bsf	INTCON,GIE
	return

; ************************************************
InPort	
	clrwdt
	banksel	Switch
	movlw	0xf0		; No key pressed
	movwf	Switch1
	movwf	Switch

	BANKSEL	PORTA
	movlw	0xE7
	movwf	PORTA
	movlw	0xFF
	movwf	PORTB

	BANKSEL	TRISB		; bank selct for TRISB
	movlw	0xf0		; initialise PORTB <4:7> as inport	
	movwf	TRISB		; PORTB <0:3> as output port

	BANKSEL	PORTB
	bcf	PORTB,0	

 	return

; ************************************************
OutPort
	clrwdt
	banksel	TRISA
	clrf	TRISA		; set PORTA as output port
	clrf	TRISB		; set PORTB as output port
	banksel	PORTA
	movlw	0xE7
	movwf	PORTA
	movlw	0xFF		; to blank the display
	movwf	PORTB
 	return

; ************************************************
InitData
	clrwdt			 
	banksel	Count0
	movlw	26h		; for 5 seconds
	movwf	Count0
	bsf	Count1,0	; to come out from loop in int.
	clrf	Neg
	movlw	0xf0		; no key is pressed
	movwf	Switch
	clrf	TEMPFlag

	return

; ************************************************
InitTmr0
	clrwdt
	banksel	OPTION_REG
	movlw	07		; TIMER0, 1:256 Prescaler
	movwf	OPTION_REG

	BANKSEL	TMR0
	clrf	TMR0
	clrf	INTCON
	bsf	INTCON,GIE
    	bsf	INTCON,T0IE	; TMR0 on	
	return

; ************************************************ 

; this routine checks if the key pressed.
; four keys are connected to output of PORTB <7:4>
; if the key is pressed, the value will be stored 
; in SW and SW1 register
ScanKey
	clrwdt
	banksel	Switch
	movlw	0xf0
	movwf	Switch		; check for four keys	
	andwf	PORTB,w
	movwf	Switch
	sublw	0xf0
	btfsc	STATUS,Z		
	return			; key is not pressed
CHANGE				; key is pressed
	movf	Switch,w
	movwf	Switch1
	return

; ************************************************
IntDelay
	clrwdt
	movlw	0X10		;	
	movwf	DB1
	movwf	DB3
DELY2	decfsz	DB3,F
	goto	DELY2
	movlw	0xff
	movwf	DB2
	decfsz	DB1,f
	goto	DELY2
	return

; ************************************************
Delay1
	clrwdt
	movlw	0X60		;
	movwf	DB2
DLY1	decfsz	DB2,F
	goto	DLY1
	return

; ************************************************
Delay_1
	clrwdt
	movlw	0X60		;
	movwf	DB1
	movwf	DB3
DLY2	decfsz	DB3,F
	goto	DLY2
	movlw	0xff
	movwf	DB2
	decfsz	DB1,f
	goto	DLY2
	return

; ************************************************
Delay20				 
	clrwdt
	movlw	.20		;
	movwf	DB1
DLAY1	clrwdt
	clrf	DB2
	decfsz	DB1,f
	goto	DLAY2
	retlw	0
DLAY2	clrwdt
	decfsz	DB2,f
	goto	DLAY1
	return

; ************************************************


 	end

