; *********************************************************************************

; Software License Agreement

; The software supplied herewith by Microchip Technology Incorporated (the Company) is 
; intended and supplied to you, the Com-panys customer, for use solely and exclusively
; on Microchip products.
; The software is owned by the Company and/or its supplier, and is protected under 
; applicable copyright laws. All rights are reserved. Any use in violation of the
; foregoing restrictions may subject the user to criminal sanctions under applicable 
; laws, as well as to civil liability for the breach of the terms and conditions of 
; this license.
; THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES, WHETHER EXPRESS, 
; IMPLIED OR STATU-TORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF 
; MERCHANTABILITY AND FITNESS FOR A PARTICU-LAR PURPOSE APPLY TO THIS SOFTWARE. 
; THE COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL
; OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

; *********************************************************************************

; device	: PIC16F84A

; ************************************************
	#include <p16F84A.inc>


	list  P=PIC16F84A
	errorlevel -302
; ************************************************

; Load Configuration bit Code protect off,WDT on, 
; Poweron timer on, XT oscillator

 __CONFIG  _CP_OFF & _WDT_ON & _PWRTE_ON & _XT_OSC 

; ************************************************
STARTUP CODE	0x00		
	goto 	STart

INTERRUPT CODE	0x04			
	goto	IntSrv
; ************************************************
	#include <data.h>

PROGRAM CODE
STart

	banksel	WDTCOUNT_HI
	btfsc	CheckCalH	; Check if the program is in High Calibration??
	call	CalForHi

	btfsc	CheckCalL	; Check if the program is in Low Calibration??
	call	CalForLow	

; ************************************************

	movf	WDTCOUNT_HI,w	; (WDTCOUNT_HI:WDTCOUNT_LO)-final value of 
	movwf	TEMPH		; 16-bit WDT counter
	movf	WDTCOUNT_LO,w	; (TEMPH:TEMPL)- value for calculation of temperature
	movwf	TEMPL
	call	InitData		; initailise the variables
	call	InPort			; configure PORTB 
REFRESH
	btfss	STATUS,NOT_TO		; 1. if reset by Power-on Reset,  
	goto	MeasureTemp		; then check for new WDT count.
					; 2. if reset by WDT Reset,  
					; then measure temperature by new count
	btfsc	SET			; Check for SET key 
	goto	WDTclr
	
	btfss	SET			; check key to release	
	goto	$-1

	call	CalibrateSystem		; for calibration of the Temperature;

; *********Close loop to get proper count*********
WDTclr	
	call	InitWDT			; Configure Prescaler for WDT in option reg.
	clrf	WDTCOUNT_HI		; 16 bit count for WDT time-out period
	clrf	WDTCOUNT_LO		; 
	clrwdt				; clear Watch Dog Timer
WDTLoop
	incfsz	WDTCOUNT_LO,f		; increment of lower 8 bit of WDT Time-out count
	goto	CalWDT
	incf	WDTCOUNT_HI,f		; increment of Upper 8 bit of WDT Time-out count
CalWDT
	goto	WDTLoop
; ************************************************

MeasureTemp
	call	EEDefault		; read the Hi & Lo calibration temperature values
					; from EEPROM for calculation of present temperature
	call	TempCalc		; calculate the temperature in centigrades
	call	InitTmr0		; initialise TMR0 as interrupt 

CheckSec
	clrwdt
	banksel	Count0
	btfsc	SecBit
	goto	OverSec

	btfsc	TEMP			; TEMP key is pressed
	goto	CheckSec
	btfss	TEMP
	goto	$-1
	call	DispTemp		; display temperature in Centigrades
	goto	CheckSec		; 

OverSec 
	bcf	SecBit	
	banksel	INTCON
	bcf	INTCON,T0IE		; disable the timer interrupt
	bsf	INTCON,RBIE		; enable the RB PORT change interrupt
	call	InPort			; initialise PortB <4:8> as inport and 

Slip	sleep				; enable scan key line to wake up from the 
	nop				; sleep, if key is pressed
	goto	WDTclr			; Update the WDT count for new temperature

; ************************************************

; interrupt routine

IntSrv
	clrwdt
	movwf	W_TEMP		; save w-register and 
	swapf	STATUS,W	; STATUS register
	movwf	STATUS_TEMP
	
	btfsc	INTCON,RBIF	; interrupt occured by port B 
	goto	KeyProc		; change interrupt

	btfss	INTCON,T0IF
	goto	RetStatus

	bcf	INTCON,T0IF	; interrupt occured by TIMER0
	decfsz	Count0,f	; for delay
	goto	RetStatus

	decfsz	Count1,f	; for delay
	goto	RetStatus

	bsf	SecBit
	goto	RetStatus

KeyProc
	bcf	INTCON,RBIF	; clear RB port change interrupt flag bit
	call	ScanKey		; read PORTB to end mismatch condition

RetStatus
	swapf	STATUS_TEMP,W	; restore w-register
	movf	STATUS,w	; STATUS register
	swapf	W_TEMP,f
	swapf 	W_TEMP,w
	retfie

; ************************************************
; This routine displays the temperature in Centigrade 
; after TEMP key is pressed
DispTemp
	clrwdt
	call	OutPort		; initalise the port B as outport
	movf	TempC,w		; temperature in Centigrade
	call	DisplayDigit	; Display Temperature on 7-segment LED display
	return

; ************************************************
; This routine displays three digits on seven segment LEDs
 
DisplayDigit
	clrwdt
	call	HexToBcd
	movf	LSD,w
	call	TABLE
	movwf	PORTB
	bcf	DIGIT0		; LSB  
	call	IntDelay
	bsf	DIGIT0

	movf	MSD,w
	call	TABLE
	movwf	PORTB
	bcf	DIGIT1		; MSB 
	call	IntDelay	 
	bsf	DIGIT1

	movf	USD,W		; sign display
	andlw	0xff
	btfsc	STATUS,Z
	goto	NoDispU
	movf	USD,w		; USB
	call	TABLE
	movwf	PORTB
	bcf	DIGIT2
	call	IntDelay
	bsf	DIGIT2

NoDispU	
	btfss	Neg,0		; if value is not -ve then return
	goto	LastClr
NegSign
	movlw	B'01111111'	; -ve SIGN will be displayed
	movwf	PORTB
	bcf	DIGIT2
	call	IntDelay	; 
LastClr
	bsf	DIGIT0
	bsf	DIGIT1
	bsf	DIGIT2
 return

; ************************************************

; this routine converts the hex number into BCD as 3 digit number

HexToBcd
	clrwdt			
	clrf	USD
	clrf	MSD
	movwf	LSD	
BCD1	movlw	.10
	subwf	LSD,W
	btfss	STATUS,C
	goto	OVER_1
	movwf	LSD		; one's digit
	incf	MSD,F
	goto	BCD1
OVER_1	
	movlw	.10
	subwf	MSD,W
	btfss	STATUS,C
	goto	OVER
	movwf	MSD		; ten's digit
	incf	USD,f		; hundred's digit
	goto	OVER_1
OVER	retlw	0

; ************************************************
; 8 bit pattern to display digits on 7 segment LED display
TABLE
					; Table for 0 to 9 display
	addwf	PCL,f			; 
	retlw	B'10000001'		; 0
	retlw	B'11110011'		; 1
	retlw	B'01001001'		; 2
	retlw	B'01100001'		; 3
	retlw	B'00110011'		; 4
	retlw	B'00100101'		; 5
	retlw	B'00000101'		; 6
	retlw	B'11110001'		; 7
	retlw	B'00000001'		; 8
	retlw	B'00110001'		; 9


; ************************************************
; Write Default =1 in EEPROM location of 0x03 
SetDefault
	clrwdt
	banksel	Default
	bsf	Default,0	; write the value in EEPROM
	movlw	0x03		; location of EEPROM
	banksel	EEADR
	movwf	EEADR
	movf	Default,w
	banksel	EEDATA
	movwf	EEDATA
	call	WriteEeprom
	return
; ************************************************
; Write Default =0 in EEPROM location of 0x03
ClrDefault
	clrwdt
	banksel	Default
	clrf	Default		; write the value in EEPROM
	movlw	0x03		; location of EEPROM
	banksel	EEADR
	movwf	EEADR
	movf	Default,w
	banksel	EEDATA
	movwf	EEDATA
	call	WriteEeprom
	return

; ************************************************

; This routine is to set the HI and LO value of the temperature for
; calibration of the system

CalibrateSystem	

CalibrationPt
	clrwdt
	banksel	Count1	
	movlw	0x03		; 
	movwf	Count1
	movlw	0x97
	movwf	Count0
	call	InitTmr0
TimeOut
	btfsc	SecBit		; over???
	return			; yes
	call	InPort
	btfss	TEMP		; if TEMP key is pressed, clear Default flag
	goto	DoTempKey

; display "HI" and high calibration temp alternately ****************

	movlw	0x40		; time delay to show 'HI' on the display  
	movwf	C1		; 
disp1	call	OutPort
	movlw	0xf3		; for display 'I'
	movwf	PORTB
	bcf	DIGIT0
	call	IntDelay	; delay 
	bsf	DIGIT0
	movlw	0x13		; for display 'H'
	movwf	PORTB
	bcf	DIGIT1
	call	IntDelay
	bsf	DIGIT1		
	decfsz	C1,f	
	goto	disp1
	clrwdt
	call	Delay_1
DISP_hi	
	movlw	0x30		; to display hi value for some delay			
	movwf	C2
DisLimit
	movf	TEMP_HI,w
	call	DisplayDigit
	decfsz	C2,f	
	goto	DisLimit
	clrwdt			
	call	Delay_1

	call	InPort
	btfss	UP			;UP key
	goto	IncDigit

	btfss	DOWN			;DOWN key
	goto	DecDigit	
	btfss	SET			;SET key
	goto	HI_out
	goto	TimeOut	

IncDigit
	btfss	UP
	goto	$-1
	clrwdt				;
	movf	TEMP_HI,w		; check for 50
	sublw	0x32
	btfss	STATUS,Z
	incf	TEMP_HI,f
	call	Delay1
	bsf	Default,0
	goto	DISP_hi	
	
DecDigit
	btfss	DOWN
	goto	$-1
	clrwdt				;
	movf	TEMP_HI,w
	iorlw	00			; to set zero flag
	btfss	STATUS,Z
	decf	TEMP_HI,f
	call	Delay1
	bsf	Default,0
	goto	DISP_hi		
HI_out
	bsf	CheckCalH
	return
	
CalForHi
	bcf	CheckCalH	
	clrwdt				;
	movf	WDTCOUNT_HI,w		; 
	movwf	WDT_H1	
	movf	WDTCOUNT_LO,w		; 
	movwf	WDT_H0	

;******************EEPROM write************************
	clrf	EEADR
	movlw	TEMP_HI			;
	movwf	FSR
	movlw	0x03
	movwf	C3
LOOP1
	clrwdt
	movf	INDF,w
	movwf	EEDATA
	call	WriteEeprom
	incf	EEADR,f
	incf	FSR,F
	decfsz	C3,f
	goto	LOOP1
	
; *********************for LO calibration *************************
	clrwdt
	movlw	0x11		; 
	movwf	Count1
	movlw	0xF3
	movwf	Count0
	call	InitTmr0
TMOutL
	btfsc	SecBit		; over???
	return			; yes
	call	InPort
	btfss	TEMP		; if TEMP key is pressed, clear Default flag
	goto	DoTempKey

lo_set
	movlw	0x40
	movwf	C1
disp1_l	
	clrwdt			;
	call	OutPort
	movlw	0x81		;for O
	movwf	PORTB
	bcf	DIGIT0
	call	IntDelay
	bsf	DIGIT0
	movlw	0x8f		; for L
	movwf	PORTB
	bcf	DIGIT1
	call	IntDelay
	bsf	DIGIT1
	decfsz	C1,f		
	goto	disp1_l
	clrwdt			;
	call	Delay_1
DISP_lo	
	clrwdt				;
	call	OutPort
	movlw	0x30
	movwf	C1
Dis_lo
	movf	TEMP_LO,w
	call	DisplayDigit
	clrwdt				;
	decfsz	C1,f
	goto	Dis_lo
	call	Delay_1
	clrwdt				;
	call	InPort
	clrwdt
	btfss	UP		;UP key
	goto	inc_digit
	btfss	DOWN		;DOWN key
	goto	dec_digit	
	btfss	SET		;SET key
	goto	LO_out
	goto	TMOutL
inc_digit
	btfss	UP
	goto	$-1	
	clrwdt				;
	movf	TEMP_LO,w
	sublw	0x32
	btfss	STATUS,Z
	incf	TEMP_LO,f
	call	Delay1
	bsf	Default,0
	goto	DISP_lo	
dec_digit
	btfss	DOWN
	goto	$-1
	clrwdt				;
	movf	TEMP_LO,w
	iorlw	00
	btfss	STATUS,Z
	decf	TEMP_LO,f
	call	Delay1
	bsf	Default,0
	goto	DISP_lo	
LO_out
	bsf	CheckCalL
	return

CalForLow
	bcf	CheckCalL
	clrwdt				;
	movf	WDTCOUNT_HI,w
	movwf	WDT_L1
	movf	WDTCOUNT_LO,w
	movwf	WDT_L0
	clrwdt				;
; *****************EEPROM write************************
	movlw	0x03
	movwf	EEADR
	movlw	0x04
	movwf	C4
	movlw	Default		; 
	movwf	FSR
LOOP_1
	clrwdt
	movf	INDF,w
	movwf	EEDATA
	call	WriteEeprom
	incf	EEADR,f
	incf	FSR,F
	decfsz	C4,f
	goto	LOOP_1
; ****************************************************

	call	SetDefault
	return

DoTempKey
	btfss	TEMP			;wait to release the key
	goto	$-1
	call	ClrDefault
	return


; *********************************************************************************

 END
	
 


